/* This program converts the analog inputs from the potentiometer,
 * light sensor, and temperature sensor on the EduPad.
 *
 * ADC is configured to scan three channels 5, 6, and 7. The 
 * conversion results are read and stored in variables.
 * Switches SW4 and SW5 are used to select the result for display
 * on the LEDs.
 * SW4 SW5 Display
 *  0   0  potentiometer
 *  0   1  photo sensor
 *  1   0  temperature sensor
 *  1   1  no output
 *
 * Connections between SAMD21 Xplained Pro and EduPad:
 *
 * Function     Pin name      Xplained pin   EduPad pin
 * AIN[5]       PA05          1-15           H4.16        Potentiometer
 * AIN[6]       PA06          1-16           H4.14        Light sensor
 * AIN[7]       PA07          1-18           H4.11        Temp senor
 * S4           PA18          2-15           H4.8
 * S5           PA19          2-17           H4.6
 * LED0         PB00          1-3            H4.5
 * LED1         PB01          1-4            H4.7
 * LED2         PB02          1-7            H5.4
 * LED3         PB03          1-8            H5.5 
 * 5V           5.0V          PWR-3          H4.2
 * ground       GND           1-19           H4.4
 *
 * Tested with Atmel Studio 7 v7.0.1006 and Keil MDK-ARM v5.21a
 */

#include "samd21.h"

unsigned char* ARRAY_PORT_PINCFG0 = (unsigned char*)&REG_PORT_PINCFG0;
unsigned char* ARRAY_PORT_PMUX0 = (unsigned char*)&REG_PORT_PMUX0;

int main (void) {
    int result, pot, light, temp;

    REG_PORT_DIR1 |= 0x0000000F;        /* make PB03-PB00 output */

    REG_PORT_DIRCLR0 = 0x00030000;      /* make PA16-17 input for SW0 */
    ARRAY_PORT_PINCFG0[16] |= 6;        /* enable PA16 input buffer with pull */
    ARRAY_PORT_PINCFG0[17] |= 6;        /* enable PA17 input buffer with pull */
    REG_PORT_OUTCLR0 = 0x00030000;      /* make PA16-17 pull-up */
    
    REG_PM_APBCMASK |= 0x10000;         /* enable bus clock for ADC */
    REG_GCLK_CLKCTRL = 0x401E;          /* GCLK0 to ADC */
    REG_ADC_SAMPCTRL = 5;               /* sampling time 5 clocks */
    ARRAY_PORT_PINCFG0[3] |= 1;         /* Use PMUX for PA03 */
    ARRAY_PORT_PMUX0[1] = 0x10;         /* PA03 = VREFA */
    REG_ADC_REFCTRL = 3;                /* Use VREFA */
    REG_ADC_INPUTCTRL = 0x00021805;     /* V- = GND; V+ = AIN5, 6, 7 */
    ARRAY_PORT_PINCFG0[5] |= 1;         /* Use PMUX for PA05 */
    ARRAY_PORT_PMUX0[2] = 0x10;         /* PA05 = AIN5 */
    ARRAY_PORT_PINCFG0[6] |= 1;         /* Use PMUX for PA06 */
    ARRAY_PORT_PINCFG0[7] |= 1;         /* Use PMUX for PA07 */
    ARRAY_PORT_PMUX0[3] = 0x11;         /* PA06 = AIN6, PA07 = AIN7 */
    REG_ADC_CTRLA = 2;                  /* enable ADC */

    while (1) {
        REG_ADC_INPUTCTRL &= ~0x00F00000; /* start a scan */
        REG_ADC_SWTRIG = 2;             /* start a conversion */
        while(!(REG_ADC_INTFLAG & 1));  /* wait for conv complete */
        pot = REG_ADC_RESULT;           /* read conversion result */
        REG_ADC_SWTRIG = 2;             /* start a conversion */
        while(!(REG_ADC_INTFLAG & 1));  /* wait for conv complete */
        light = REG_ADC_RESULT;         /* read conversion result */
        REG_ADC_SWTRIG = 2;             /* start a conversion */
        while(!(REG_ADC_INTFLAG & 1));  /* wait for conv complete */
        temp = REG_ADC_RESULT;          /* read conversion result */
       
        switch(REG_PORT_IN0 & 0x00030000) {
            case 0x00000000:
                result = pot >> 8;
                break;
            case 0x00010000:
                result = light >> 8;
                break;
            case 0x00020000:
                result = temp;
                break;
            case 0x00030000:
                result = 0;
        }
        
        REG_PORT_OUTCLR1 = 0x000F;      /* turn off all LEDs */
        REG_PORT_OUTSET1 = result & 0x000F;  /* turn on LEDs */
    }
}
