/* OLED_SSD1306 via I2C
 *
 * This program communicate with the OLED_SSD1306 via I2C.
 * It writes "ABC" on the display and fills the rest of the
 * display with a checker board pattern.
 *
 * Connections between SAMD21 Xplained Pro and EduPad:
 * Function     Pin name      Xplained pin   EduPad pin
 * SCL          PA17          2-15           H4.17
 * SDA          PA16          2-17           H4.19
 * ground       GND           1-19           H4.4
 *
 * Tested with Atmel Studio 7 v7.0.1006 and Keil MDK-ARM v5.21a
 */

#include "samd21.h"

#define SLAVE_ADDR 0x3C     // 0111 100.
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define SCREEN_PAGE_NUM (SCREEN_HEIGHT / 8)

void SSD1306_init(void);
void SSD1306_data(char data);
void SSD1306_command(char command);
void I2C2_init(void);
int I2C2_byteWrite(unsigned char saddr, unsigned char maddr, unsigned char data);
void delayMs(int n);

char imagebuf[SCREEN_PAGE_NUM][SCREEN_WIDTH];

const static char font_table[][6] = {    /* sample font table */
    {0x7e, 0x11, 0x11, 0x11, 0x7e, 0},  /* A */
    {0x7f, 0x49, 0x49, 0x49, 0x36, 0},  /* B */
    {0x3e, 0x41, 0x41, 0x41, 0x22, 0}}; /* C */

int main(void) {
    int i, j, k;

    I2C2_init();
    SSD1306_init();

    SSD1306_command(0x22);              /* set page */
    SSD1306_command(0x00);              /* starting page */
    SSD1306_command(0x00);              /* end page */
    SSD1306_command(0x21);              /* set column */
    SSD1306_command(0);                 /* starting column */
    SSD1306_command(SCREEN_WIDTH-1);    /* end column */

    /* write ABC at the beginning of first line */
    for (j = 0; j < 3; j++)
        for (i = 0; i < 6; i++)
            SSD1306_data(font_table[j][i]);

    /* clear the rest of the line */
    for (i = 0; i < SCREEN_WIDTH-3*6; i++)
        SSD1306_data(0);

    for (i = 1; i < SCREEN_PAGE_NUM; i++) {
        SSD1306_command(0x22);              /* set pages */
        SSD1306_command(i);                 /*   starting page */
        SSD1306_command(i);                 /*   end page */
        SSD1306_command(0x21);              /* set columns */
        SSD1306_command(0);                 /*   starting column */
        SSD1306_command(SCREEN_WIDTH-1);    /*   end column */

        /* make checker board pattern */
        for (j = 0; j < SCREEN_WIDTH/8; j++) {
            for (k = 0; k < 4; k++)
                SSD1306_data(0xF0);

            for (k = 0; k < 4; k++)
                SSD1306_data(0x0F);
        }
    }

    for (;;) {  /* infinite loop */
    }
}

const static char init_seq[] = {
    0xae,       // turn off oled panel
    0x00,       // set low column address
    0x10,       // set high column address
    0x40,       // set start line address
    0x20, 0x02, // page addressing mode
    0xc8,       // top-down segment (4th quadrant)
    0x81,       // set contrast control register
    0xcf, 0xa1, // set segment re-map 95 to 0
    0xa6,       // set normal display
    0xa8,       // set multiplex ratio(1 to 64)
    0x3f,       // 1/64 duty
    0xd3,       // set display offset
    0x00,       // not offset
    0xd5,       // set display clock divide ratio/oscillator frequency
    0x80,       // set divide ratio
    0xd9,       // set pre-charge period
    0xf1, 0xda, // set com pins hardware configuration
    0x12, 0xdb, // set vcomh
    0x40, 0x8d, // set Charge Pump enable/disable
    0x14,       // set(0x10) disable
    0xaf        // turn on oled panel
};

void SSD1306_init(void) {
    int i;

    delayMs(100);

	for (i = 0; i < sizeof(init_seq); i++)
        SSD1306_command(init_seq[i]);
}

void SSD1306_data(char data) {
    I2C2_byteWrite(SLAVE_ADDR, 0xC0, data);
}

void SSD1306_command(char command) {
    I2C2_byteWrite(SLAVE_ADDR, 0x80, command);
}

void I2C2_init(void) {
    unsigned char* ARRAY_PORT_PINCFG0 = (unsigned char*)&REG_PORT_PINCFG0;
    unsigned char* ARRAY_PORT_PMUX0 = (unsigned char*)&REG_PORT_PMUX0;

    REG_PM_APBCMASK |= 0x00000008;          /* SERCOM1 bus clock */
    GCLK->CLKCTRL.reg = 0x4015;             /* SERCOM1 core clock */

    ARRAY_PORT_PINCFG0[16] |= 1;            /* allow pmux to set PA16 pin configuration */
    ARRAY_PORT_PINCFG0[17] |= 1;            /* allow pmux to set PA17 pin configuration */
    ARRAY_PORT_PMUX0[8] = 0x22;             /* PA16 = SDA, PA17 = SCL */

    REG_SERCOM1_I2CM_CTRLA = 1;             /* reset SERCOM1 */
    while (REG_SERCOM1_I2CM_CTRLA & 1) {}   /* wait for reset to complete */
    REG_SERCOM1_I2CM_CTRLA = 0x14;          /* master mode */
    REG_SERCOM1_I2CM_BAUD = 0;              /* 1MHz main clock -> ~100KHz I2C clock */
    REG_SERCOM1_I2CM_CTRLA |= 2;            /* enable SERCOM1 */
    REG_SERCOM1_I2CM_STATUS = 0x10;         /* force idle */
}

int I2C2_byteWrite(unsigned char saddr, unsigned char maddr, unsigned char data) {
    while((REG_SERCOM1_I2CM_STATUS & 0x30) != 0x10) ; /* wait until idle */

    REG_SERCOM1_I2CM_ADDR = saddr << 1;         /* send address */
    while((REG_SERCOM1_I2CM_INTFLAG & 1) == 0); /* wait until saddr sent */

    REG_SERCOM1_I2CM_DATA = maddr;              /* send memory address */
    while((REG_SERCOM1_I2CM_INTFLAG & 1) == 0); /* wait until maddr sent */

    REG_SERCOM1_I2CM_DATA = data;               /* send data */
    while((REG_SERCOM1_I2CM_INTFLAG & 1) == 0); /* wait until data sent */

    REG_SERCOM1_I2CM_CTRLB |= 0x30000;          /* issue a stop */

    return 0;
}

// delay n milliseconds (1MHz CPU clock)
void delayMs(int n) {
    int i;
	for (; n > 0; n--)
		for (i = 0; i < 199; i++)
			__asm("nop");
}
