/* Read register 0 (second count) of DS3231
 *
 * This program reads register at address 0 in the DS3231, the second count, 
 * repetitively and dump the data to the LEDs.  Since the second count uses 
 * binary coded decimal format, the LEDs will count up from 0 to 9 in binary
 * and wrap around to 0.
 * 
 * On EduPad board, PB03-PB00 are connected to the four LEDs.
 *
 * Connections between SAMD21 Xplained Pro and EduPad:
 * Function     Pin name      Xplained pin   EduPad pin
 * SCL          PA17          2-15           H4.17
 * SDA          PA16          2-17           H4.19
 * LED0         PB00          1-3            H4.5
 * LED1         PB01          1-4            H4.7
 * LED2         PB02          1-7            H5.4
 * LED3         PB03          1-8            H5.5 
 * ground       GND           1-19           H4.4
 *
 * Tested with Atmel Studio 7 v7.0.1006 and Keil MDK-ARM v5.21a
 */

#include "samd21.h"

void delayMs(int n);
void I2C2_init(void);
int I2C2_byteRead(unsigned char saddr, unsigned char maddr, unsigned char* data);

#define SLAVE_ADDR 0x68     // 1101 000.

int main (void) {
    unsigned char readback;
    unsigned char error;

    REG_PORT_DIR1 |= 0x0000000F;       /* make PB03-PB00 output */

    I2C2_init();
    
    for (;;) {
        // use second count to turn LEDs on or off
        error = I2C2_byteRead(SLAVE_ADDR, 0, &readback);
        if (error)
            for(;;) ;   // replace with error handling
        REG_PORT_OUTCLR1 = 0x000F;  /* turn off all LEDs */
        REG_PORT_OUTSET1 = readback;  /* turn on LEDs */
        delayMs(200);
    }
}

void I2C2_init(void) {
    unsigned char* ARRAY_PORT_PINCFG0 = (unsigned char*)&REG_PORT_PINCFG0;
    unsigned char* ARRAY_PORT_PMUX0 = (unsigned char*)&REG_PORT_PMUX0;

    REG_PM_APBCMASK |= 0x00000008;          /* SERCOM1 bus clock */
    GCLK->CLKCTRL.reg = 0x4015;             /* SERCOM1 core clock */

    ARRAY_PORT_PINCFG0[16] |= 1;            /* allow pmux to set PA16 pin configuration */
    ARRAY_PORT_PINCFG0[17] |= 1;            /* allow pmux to set PA17 pin configuration */
    ARRAY_PORT_PMUX0[8] = 0x22;             /* PA16 = SDA, PA17 = SCL */

    REG_SERCOM1_I2CM_CTRLA = 1;             /* reset SERCOM1 */
    while (REG_SERCOM1_I2CM_CTRLA & 1) {}   /* wait for reset to complete */
    REG_SERCOM1_I2CM_CTRLA = 0x14;          /* master mode */
    REG_SERCOM1_I2CM_BAUD = 0;              /* 1MHz main clock -> ~100KHz I2C clock */
    REG_SERCOM1_I2CM_CTRLA |= 2;            /* enable SERCOM1 */
    REG_SERCOM1_I2CM_STATUS = 0x10;         /* force idle */
}

int I2C2_byteRead(unsigned char saddr, unsigned char maddr, unsigned char* data) {
    while((REG_SERCOM1_I2CM_STATUS & 0x30) != 0x10) ; /* wait until idle */

    REG_SERCOM1_I2CM_ADDR = saddr << 1;         /* send slave address */
    while((REG_SERCOM1_I2CM_INTFLAG & 1) == 0); /* wait until saddr sent */

    REG_SERCOM1_I2CM_DATA = maddr;              /* send memory address */
    while((REG_SERCOM1_I2CM_INTFLAG & 1) == 0); /* wait until maddr sent */

    REG_SERCOM1_I2CM_ADDR = (saddr << 1) | 1;   /* send restart with slave address */
    while((REG_SERCOM1_I2CM_INTFLAG & 2) == 0); /* wait until data received */

    *data = REG_SERCOM1_I2CM_DATA;              /* read data */

    REG_SERCOM1_I2CM_CTRLB |= 0x40000;          /* generate nack */
    REG_SERCOM1_I2CM_CTRLB |= 0x30000;          /* issue a stop */
    return 0;
}

/* millisecond delay based on 1 MHz system clock */
void delayMs(int n) {
    int i;
    for (; n > 0; n--)
        for (i = 0; i < 199; i++)
            __asm("nop");
}
